package edu.mit.harold.icollections;

import java.util.Collection;




/**
 *
 * A static convenience class for creating efficient immutable sets.
 * <p>
 * This class simply creates MapISets backed by HashTreeIMaps.
 * 
 * @author harold
 */
public final class HashTreeISet {
	// not instantiable (or subclassable):
	private HashTreeISet() {}
	
	private static final MapISet<Object> EMPTY = MapISet.from(HashTreeIMap.empty());

	/**
	 * @param <E>
	 * @return an empty set
	 */
	@SuppressWarnings("unchecked")
	public static <E> MapISet<E> empty() {
		return (MapISet<E>)EMPTY; }
	
	/**
	 * @param <E>
	 * @param e
	 * @return empty().plus(e)
	 */
	public static <E> MapISet<E> singleton(final E e) {
		return HashTreeISet.<E>empty().plus(e); }
	
	/**
	 * @param <E>
	 * @param list
	 * @return empty().plusAll(map)
	 */
	public static <E> MapISet<E> from(final Collection<? extends E> list) {
		return HashTreeISet.<E>empty().plusAll(list); }
}
