package edu.mit.harold.icollections.tests;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import edu.mit.harold.icollections.ConsIStack;
import edu.mit.harold.icollections.HashTreeIBag;
import edu.mit.harold.icollections.HashTreeIMap;
import edu.mit.harold.icollections.HashTreeISet;
import edu.mit.harold.icollections.IBag;
import edu.mit.harold.icollections.ICollection;
import edu.mit.harold.icollections.IMap;
import edu.mit.harold.icollections.ISet;
import edu.mit.harold.icollections.IStack;
import edu.mit.harold.icollections.IVector;
import edu.mit.harold.icollections.IntTreeIMap;
import edu.mit.harold.icollections.TreeIVector;


@SuppressWarnings("unchecked")
public class Benchmarks {
	
	private static final String METHOD = "%65s = ",
		TIME = "%5d ms";

	/**
	 * 
	 * Runs benchmarks() on a list of linear keys and a list of random keys.
	 * 
	 * @param args
	 */
	public static void main(final String[] args) {
		// make an array of linear indices
		int n = 10000;
		Object[] list = new Object[n];
		for(int i=0; i<n; i++) list[i] = i;
		System.out.println("***Benchmarks for "+n+" linear keys:***");
		benchmarks(list);
		
		System.out.println();

		Random r = new Random();
		for(int i=0; i<n; i++) list[i] = r.nextInt();
		System.out.println("***Benchmarks for "+n+" random keys:***");
		benchmarks(list);
	}
	
	/**
	 * 
	 * Prints to stdout the times of various operations on LinkedLists,
	 * ConsIStacks, ArrayLists, TreeIVectors, HashSets, HashTreeISets,
	 * HashTreeIBags, HashMaps, HashTreeIMaps, and IntTreeIMaps.
	 * 
	 * @param list An array of keys
	 */
	public static void benchmarks(final Object[] list) {
		List linkedList = new LinkedList();
		IStack consIStack = ConsIStack.empty();

		List arrayList = new ArrayList();
		IVector treeIVector = TreeIVector.empty();

		Set hashSet = new HashSet();
		ISet hashTreeISet = HashTreeISet.empty();
		IBag hashTreeIBag = HashTreeIBag.empty();
		
		Map hashMap = new HashMap();
		IMap hashTreeIMap = HashTreeIMap.empty();
		IMap intTreeIMap = IntTreeIMap.empty();

		System.out.println("Sequential Lists:");
		collectionAdd(linkedList, list);
		consIStack = (IStack)iCollectionPlus(consIStack, list);
		collectionContains(linkedList, list);
		collectionContains(consIStack, list);
		collectionNotContains(linkedList, list.length);
		collectionNotContains(consIStack, list.length);
		listGet(linkedList);
		listGet(consIStack);
		collectionIterator(linkedList);
		collectionIterator(consIStack);

		System.out.println("Random-access Lists:");
		collectionAdd(arrayList, list);
		treeIVector = (IVector)iCollectionPlus(treeIVector, list);
		collectionContains(arrayList, list);
		collectionContains(treeIVector, list);
		collectionNotContains(arrayList, list.length);
		collectionNotContains(treeIVector, list.length);
		listGet(arrayList);
		listGet(treeIVector);
		collectionIterator(arrayList);
		collectionIterator(treeIVector);

		System.out.println("Sets and Bags:");
		collectionAdd(hashSet, list);
		hashTreeISet = (ISet)iCollectionPlus(hashTreeISet, list);
		hashTreeIBag = (IBag)iCollectionPlus(hashTreeIBag, list);
		collectionContains(hashSet, list);
		collectionContains(hashTreeISet, list);
		collectionContains(hashTreeIBag, list);
		collectionNotContains(hashSet, list.length);
		collectionNotContains(hashTreeISet, list.length);
		collectionNotContains(hashTreeIBag, list.length);
		collectionIterator(hashSet);
		collectionIterator(hashTreeISet);
		collectionIterator(hashTreeIBag);

		System.out.println("Maps:");
		mapPut(hashMap, list);
		hashTreeIMap = iMapPlus(hashTreeIMap, list);
		intTreeIMap = iMapPlus(intTreeIMap, list);
		mapContainsKey(hashMap, list);
		mapContainsKey(hashTreeIMap, list);
		mapContainsKey(intTreeIMap, list);
		mapNotContainsKey(hashMap, list.length);
		mapNotContainsKey(hashTreeIMap, list.length);
		mapNotContainsKey(intTreeIMap, list.length);
	}
	
	private static ICollection iCollectionPlus(ICollection c, final Object[] list) {
		System.out.format(METHOD, c.getClass().getName()+".plus()");
		long t0 = System.currentTimeMillis();
		for(Object e : list) 
			c = c.plus(e);
		long t1 = System.currentTimeMillis();
		System.out.format(TIME, t1-t0);
		System.out.println();
		return c;
	}
	
	private static void collectionAdd(final Collection c, final Object[] list) {
		System.out.format(METHOD, c.getClass().getName()+".add()");
		long t0 = System.currentTimeMillis();
		for(Object e : list) 
			c.add(e);
		long t1 = System.currentTimeMillis();
		System.out.format(TIME, t1-t0);
		System.out.println();
	}
	
	private static IMap iMapPlus(IMap m, final Object[] list) {
		System.out.format(METHOD, m.getClass().getName()+".plus()");
		long t0 = System.currentTimeMillis();
		for(Object e : list) 
			m = m.plus(e,e);
		long t1 = System.currentTimeMillis();
		System.out.format(TIME, t1-t0);
		System.out.println();
		return m;
	}
	
	private static void mapPut(final Map m, final Object[] list) {
		System.out.format(METHOD, m.getClass().getName()+".put()");
		long t0 = System.currentTimeMillis();
		for(Object e : list) 
			m.put(e,e);
		long t1 = System.currentTimeMillis();
		System.out.format(TIME, t1-t0);
		System.out.println();
	}
	
	private static void collectionContains(final Collection c, final Object[] list) {
		System.out.format(METHOD, c.getClass().getName()+".contains()==true");
		long t0 = System.currentTimeMillis();
		for(Object e : list)
			c.contains(e);
		long t1 = System.currentTimeMillis();
		System.out.format(TIME, t1-t0);
		System.out.println();
	}
	
	private static void collectionNotContains(final Collection c, final int n) {
		System.out.format(METHOD, c.getClass().getName()+".contains()==false");
		long t0 = System.currentTimeMillis();
		for(int i=0; i<n; i++)
			c.contains("not an element of c");
		long t1 = System.currentTimeMillis();
		System.out.format(TIME, t1-t0);
		System.out.println();
	}
	
	private static void mapContainsKey(final Map m, final Object[] list) {
		System.out.format(METHOD, m.getClass().getName()+".containsKey()==true");
		long t0 = System.currentTimeMillis();
		for(Object e : list)
			m.containsKey(e);
		long t1 = System.currentTimeMillis();
		System.out.format(TIME, t1-t0);
		System.out.println();
	}
	
	private static void mapNotContainsKey(final Map m, final int n) {
		System.out.format(METHOD, m.getClass().getName()+".containsKey()==false");
		long t0 = System.currentTimeMillis();
		for(int i=0; i<n; i++)
			m.containsKey("not a key of m");
		long t1 = System.currentTimeMillis();
		System.out.format(TIME, t1-t0);
		System.out.println();
	}
	
	private static void listGet(final List c) {
		System.out.format(METHOD, c.getClass().getName()+".get()");
		long t0 = System.currentTimeMillis();
		for(int i=0;i<c.size();i++)
			c.get(i);
		long t1 = System.currentTimeMillis();
		System.out.format(TIME, t1-t0);
		System.out.println();
	}
	
	private static void collectionIterator(final Collection c) {
		System.out.format(METHOD, c.getClass().getName()+".iterator()");
		long t0 = System.currentTimeMillis();
		for(@SuppressWarnings("unused") Object e : c);
		long t1 = System.currentTimeMillis();
		System.out.format(TIME, t1-t0);
		System.out.println();
	}

}
